@@ -1,6 +1,6 @@
 # vim: set fileencoding=utf-8 :
 #
-# (C) 2007,2009,2015 Guido Guenther <agx@sigxcpu.org>
+# (C) 2007,2009,2015,2017 Guido Guenther <agx@sigxcpu.org>
 #    This program is free software; you can redistribute it and/or modify
 #    it under the terms of the GNU General Public License as published by
 #    the Free Software Foundation; either version 2 of the License, or
@@ -42,25 +42,27 @@ def proxy_stdf():
     object that is not a file object (compatible) and thus causes a crash with
     Popen.
     """
-    stdout = None
-    if not hasattr(sys.stdout, 'fileno'):
-        stdout = sys.stdout
-        sys.stdout = TemporaryFile()
-    stderr = None
-    if not hasattr(sys.stderr, 'fileno'):
-        stderr = sys.stderr
-        sys.stderr = TemporaryFile()
+    tmp_stdout = sys.stdout
     try:
-        yield
+        tmp_stdout.fileno()
+    except Exception:
+        tmp_stdout = TemporaryFile()
+
+    tmp_stderr = sys.stderr
+    try:
+        tmp_stderr.fileno()
+    except Exception:
+        tmp_stderr = TemporaryFile()
+
+    try:
+        yield tmp_stdout, tmp_stderr
     finally:
-        if stdout:
-            sys.stdout.seek(0)
-            stdout.write(sys.stdout.read())
-            sys.stdout = stdout
-        if stderr:
-            sys.stderr.seek(0)
-            stderr.write(sys.stderr.read())
-            sys.stderr = stderr
+        if tmp_stdout != sys.stdout:
+            tmp_stdout.seek(0)
+            sys.stdout.write(tmp_stdout.read().decode())
+        if tmp_stderr != sys.stderr:
+            tmp_stderr.seek(0)
+            sys.stderr.write(tmp_stderr.read().decode())
 
 
 class Command(object):
@@ -70,6 +72,8 @@ class Command(object):
 
     Note that it does not do any shell quoting even with shell=True so
     you have to quote arguments yourself if necessary.
+
+    If cmd doesn't contain a path component it will be looked up in $PATH.
     """
     def __init__(self, cmd, args=[], shell=False, extra_env=None, cwd=None,
                  capture_stderr=False,
@@ -107,9 +111,10 @@ class Command(object):
         if self.shell:
             # subprocess.call only cares about the first argument if shell=True
             cmd = " ".join(cmd)
-        with proxy_stdf():
-            stdout_arg = subprocess.PIPE if self.capture_stdout else sys.stdout
-            stderr_arg = subprocess.PIPE if self.capture_stderr else sys.stderr
+        with proxy_stdf() as (stdout, stderr):
+            stdout_arg = subprocess.PIPE if self.capture_stdout else stdout
+            stderr_arg = subprocess.PIPE if self.capture_stderr else stderr
+
             try:
                 popen = subprocess.Popen(cmd,
                                          cwd=self.cwd,
@@ -119,6 +124,10 @@ class Command(object):
                                          stdout=stdout_arg,
                                          stderr=stderr_arg)
                 (self.stdout, self.stderr) = popen.communicate()
+                if self.stdout is not None:
+                    self.stdout = self.stdout.decode()
+                if self.stderr is not None:
+                    self.stderr = self.stderr.decode()
             except OSError as err:
                 self.err_reason = "execution failed: %s" % str(err)
                 self.retcode = 1
@@ -169,10 +178,10 @@ class Command(object):
         @type quiet: C{bool}
 
         >>> Command("/bin/true")(["foo", "bar"])
-        >>> Command("/foo/bar")(quiet=True)
+        >>> Command("/foo/bar")(quiet=True) # doctest: +IGNORE_EXCEPTION_DETAIL
         Traceback (most recent call last):
         ...
-        CommandExecFailed: '/foo/bar' failed: execution failed: [Errno 2] No such file or directory
+        gbp.command_wrappers.CommandExecFailed
         """
         try:
             ret = self.__call(args)
@@ -205,24 +214,24 @@ class Command(object):
         >>> Command("/foo/bar").call(["foo", "bar"]) # doctest:+ELLIPSIS
         Traceback (most recent call last):
         ...
-        CommandExecFailed: execution failed: ...
+        gbp.command_wrappers.CommandExecFailed: execution failed: ...
         >>> c = Command("/bin/true", capture_stdout=True,
         ...             extra_env={'LC_ALL': 'C'})
         >>> c.call(["--version"])
         0
-        >>> c.stdout.decode('utf-8').startswith('true')
+        >>> c.stdout.startswith('true')
         True
         >>> c = Command("/bin/false", capture_stdout=True,
         ...             extra_env={'LC_ALL': 'C'})
         >>> c.call(["--help"])
         1
-        >>> c.stdout.decode('utf-8').startswith('Usage:')
+        >>> c.stdout.startswith('Usage:')
         True
         """
+        ret = 1
         try:
             ret = self.__call(args)
         except OSError:
-            ret = 1
             raise CommandExecFailed(self.err_reason)
         finally:
             if ret and not quiet:
