@@ -25,21 +25,23 @@ from gbp.deb.policy import DebianPkgPolicy
 
 
 class DscFile(object):
-    """Keeps all needed data read from a dscfile"""
+    """Keeps data read from a dscfile"""
     compressions = r"(%s)" % '|'.join(DebianUpstreamSource.known_compressions())
     pkg_re = re.compile(r'Source:\s+(?P<pkg>.+)\s*')
     version_re = re.compile(r'Version:\s((?P<epoch>\d+)\:)?'
-                             '(?P<version>[%s]+)\s*$'
+                            '(?P<version>[%s]+)\s*$'
                             % DebianPkgPolicy.debianversion_chars)
     tar_re = re.compile(r'^\s\w+\s\d+\s+(?P<tar>[^_]+_[^_]+'
-                         '(\.orig)?\.tar\.%s)' % compressions)
+                        '(\.orig)?\.tar\.%s)$' % compressions)
     add_tar_re = re.compile(r'^\s\w+\s\d+\s+(?P<tar>[^_]+_[^_]+'
-                            '\.orig-(?P<dir>[a-z0-9-]+)\.tar\.%s)' % compressions)
+                            '\.orig-(?P<dir>[a-zA-Z0-9-]+)\.tar\.%s)$' % compressions)
     diff_re = re.compile(r'^\s\w+\s\d+\s+(?P<diff>[^_]+_[^_]+'
-                          '\.diff.(gz|bz2))')
+                         '\.diff.(gz|bz2))$')
     deb_tgz_re = re.compile(r'^\s\w+\s\d+\s+(?P<deb_tgz>[^_]+_[^_]+'
-                             '\.debian.tar.%s)' % compressions)
+                            '\.debian.tar.%s)$' % compressions)
     format_re = re.compile(r'Format:\s+(?P<format>[0-9.]+)\s*')
+    sig_re = re.compile(r'^\s\w+\s\d+\s+(?P<sig>[^_]+_[^_]+'
+                        '\.orig(-[a-z0-9-]+)?\.tar\.%s.asc)$' % compressions)
 
     def __init__(self, dscfile):
         self.pkg = ""
@@ -51,6 +53,7 @@ class DscFile(object):
         self.upstream_version = ""
         self.native = False
         self.dscfile = os.path.abspath(dscfile)
+        sigs = []
         add_tars = []
 
         f = open(self.dscfile)
@@ -63,7 +66,7 @@ class DscFile(object):
                     self.upstream_version = "-".join(m.group('version').split("-")[0:-1])
                     self.native = False
                 else:
-                    self.native = True # Debian native package
+                    self.native = True  # Debian native package
                     self.upstream_version = m.group('version')
                 if m.group('epoch'):
                     self.epoch = m.group('epoch')
@@ -87,6 +90,10 @@ class DscFile(object):
             if m:
                 self.tgz = os.path.join(fromdir, m.group('tar'))
                 continue
+            m = self.sig_re.match(line)
+            if m:
+                sigs.append(os.path.join(fromdir, m.group('sig')))
+                continue
             m = self.diff_re.match(line)
             if m:
                 self.diff = os.path.join(fromdir, m.group('diff'))
@@ -97,26 +104,34 @@ class DscFile(object):
                 continue
         f.close()
 
+        # Source format 1.0 can have non-native packages without a Debian revision:
+        # e.g. http://snapshot.debian.org/archive/debian/20090801T192339Z/pool/main/l/latencytop/latencytop_0.5.dsc
+        if self.pkgformat == "1.0" and self.diff:
+            self.native = False
+        elif not self.native and not self.debian_version:
+            raise GbpError("Cannot parse Debian version number from '%s'" % self.dscfile)
+
         if not self.pkg:
             raise GbpError("Cannot parse package name from '%s'" % self.dscfile)
         elif not self.tgz:
             raise GbpError("Cannot parse archive name from '%s'" % self.dscfile)
         if not self.upstream_version:
             raise GbpError("Cannot parse version number from '%s'" % self.dscfile)
-        if not self.native and not self.debian_version:
-            raise GbpError("Cannot parse Debian version number from '%s'" % self.dscfile)
         self.additional_tarballs = dict(add_tars)
+        self.sigs = list(set(sigs))
 
-    def _get_version(self):
-        version = [ "", self.epoch + ":" ][len(self.epoch) > 0]
+    @property
+    def version(self):
+        version = ["", self.epoch + ":"][len(self.epoch) > 0]
         if self.native:
             version += self.upstream_version
         else:
-            version += "%s-%s" % (self.upstream_version, self.debian_version)
+            if self.debian_version != '':
+                version += "%s-%s" % (self.upstream_version, self.debian_version)
+            else:   # possible in 1.0
+                version += "%s" % self.upstream_version
         return version
 
-    version = property(_get_version)
-
     def __str__(self):
         return "<%s object %s>" % (self.__class__.__name__, self.dscfile)
 
