@@ -17,11 +17,10 @@
 """provides some debian source package related helpers"""
 
 import os
+from gbp.deb import DebianPkgPolicy as Policy
 from gbp.deb.format import DebianSourceFormat
 from gbp.deb.changelog import ChangeLog
 
-import six
-
 
 class FileVfs(object):
     def __init__(self, dir):
@@ -51,12 +50,12 @@ class DebianSource(object):
     """
     def __init__(self, vfs):
         """
-        @param vfs: a class that implemented GbpVFS interfacce or
-             a directory (which will used the DirGbpVFS class.
+        @param vfs: a class that implemented GbpVFS interface or
+             a directory (which will use the I{FileVFS} class.
         """
         self._changelog = None
 
-        if isinstance(vfs, six.string_types):
+        if isinstance(vfs, str):
             self._vfs = FileVfs(vfs)
         else:
             self._vfs = vfs
@@ -78,6 +77,14 @@ class DebianSource(object):
         except IOError as e:
             raise DebianSourceError("Failed to determine source format: %s" % e)
 
+    def is_releasable(self):
+        """
+        Check if package is releasable
+
+        Debian's current practive is to check for UNRELEASED in the distribution.
+        """
+        return self.changelog.distribution != 'UNRELEASED'
+
     @property
     def changelog(self):
         """
@@ -113,3 +120,28 @@ class DebianSource(object):
     @property
     def debian_version(self):
         return self.changelog.debian_version
+
+    def upstream_tarball_name(self, compression, component=None):
+        """
+        Possible upstream tarball name for this source package
+
+        Gives the name of the main tarball if component is None
+        """
+        if self.is_native():
+            return None
+        return Policy.build_tarball_name(self.name,
+                                         self.upstream_version,
+                                         compression=compression,
+                                         component=component)
+
+    def upstream_tarball_names(self, comp_type, components=None):
+        """
+        Possible upstream tarballs names for this source package
+
+        This includes component tarballs names.  with the given
+        component names
+        """
+        names = [self.upstream_tarball_name(comp_type)]
+        for component in (components or []):
+            names += [self.upstream_tarball_name(comp_type, c) for c in components]
+        return names
