@@ -23,19 +23,22 @@ import tempfile
 from optparse import OptionParser
 from collections import defaultdict
 
-import six
-
 import gbp.command_wrappers as gbpc
 from gbp.errors import GbpError
 from gbp.git import GitRepositoryError
 from gbp.patch_series import (PatchSeries, Patch)
 import gbp.log
-from gbp.pkg import (UpstreamSource, parse_archive_filename)
+from gbp.pkg import (UpstreamSource, Archive)
 from gbp.rpm.policy import RpmPkgPolicy
 from gbp.rpm.linkedlist import LinkedList
 from gbp.rpm.lib_rpm import librpm, get_librpm_log
 
 
+def _decode(s):
+    if s is not None:
+        return s.decode()
+
+
 class NoSpecError(Exception):
     """Spec file parsing error"""
     pass
@@ -71,8 +74,8 @@ class SrcRpmFile(object):
     @property
     def version(self):
         """Get the (downstream) version of the RPM package"""
-        version = dict(upstreamversion=self.rpmhdr[librpm.RPMTAG_VERSION],
-                       release=self.rpmhdr[librpm.RPMTAG_RELEASE])
+        version = dict(upstreamversion=self.rpmhdr[librpm.RPMTAG_VERSION].decode(),
+                       release=self.rpmhdr[librpm.RPMTAG_RELEASE].decode())
         if self.rpmhdr[librpm.RPMTAG_EPOCH] is not None:
             version['epoch'] = str(self.rpmhdr[librpm.RPMTAG_EPOCH])
         return version
@@ -80,17 +83,17 @@ class SrcRpmFile(object):
     @property
     def name(self):
         """Get the name of the RPM package"""
-        return self.rpmhdr[librpm.RPMTAG_NAME]
+        return self.rpmhdr[librpm.RPMTAG_NAME].decode()
 
     @property
     def upstreamversion(self):
         """Get the upstream version of the RPM package"""
-        return self.rpmhdr[librpm.RPMTAG_VERSION]
+        return self.rpmhdr[librpm.RPMTAG_VERSION].decode()
 
     @property
     def packager(self):
         """Get the packager of the RPM package"""
-        return self.rpmhdr[librpm.RPMTAG_PACKAGER]
+        return _decode(self.rpmhdr[librpm.RPMTAG_PACKAGER])
 
     def unpack(self, dest_dir):
         """
@@ -160,13 +163,13 @@ class SpecFile(object):
 
         # Other initializations
         source_header = self._specinfo.packages[0].header
-        self.name = source_header[librpm.RPMTAG_NAME]
-        self.upstreamversion = source_header[librpm.RPMTAG_VERSION]
-        self.release = source_header[librpm.RPMTAG_RELEASE]
+        self.name = source_header[librpm.RPMTAG_NAME].decode()
+        self.upstreamversion = source_header[librpm.RPMTAG_VERSION].decode()
+        self.release = source_header[librpm.RPMTAG_RELEASE].decode()
         # rpm-python returns epoch as 'long', convert that to string
         self.epoch = str(source_header[librpm.RPMTAG_EPOCH]) \
             if source_header[librpm.RPMTAG_EPOCH] is not None else None
-        self.packager = source_header[librpm.RPMTAG_PACKAGER]
+        self.packager = _decode(source_header[librpm.RPMTAG_PACKAGER])
         self._tags = {}
         self._special_directives = defaultdict(list)
         self._gbp_tags = defaultdict(list)
@@ -185,7 +188,7 @@ class SpecFile(object):
     def _parse_filtered_spec(self, skip_tags):
         """Parse a filtered spec file in rpm-python"""
         skip_tags = [tag.lower() for tag in skip_tags]
-        with tempfile.NamedTemporaryFile(prefix='gbp') as filtered:
+        with tempfile.NamedTemporaryFile(prefix='gbp', mode='w+') as filtered:
             filtered.writelines(str(line) for line in self._content
                                 if str(line).split(":")[0].strip().lower() not in skip_tags)
             filtered.flush()
@@ -296,7 +299,7 @@ class SpecFile(object):
             tagvalue = None
         # We don't support "multivalue" tags like "Provides:" or "SourceX:"
         # Rpm python doesn't support many of these, thus the explicit list
-        if isinstance(tagvalue, six.integer_types):
+        if isinstance(tagvalue, int):
             tagvalue = str(tagvalue)
         elif type(tagvalue) is list or tagname in self._listtags:
             tagvalue = None
@@ -313,6 +316,8 @@ class SpecFile(object):
             self._tags[tagname]['value'] = tagvalue
             self._tags[tagname]['lines'].append(linerecord)
         else:
+            if tagvalue and not isinstance(tagvalue, str):
+                tagvalue = tagvalue.decode()
             self._tags[tagname] = {'value': tagvalue, 'lines': [linerecord]}
 
         return tagname
@@ -692,7 +697,7 @@ class SpecFile(object):
             macro_line = self._content.insert_after(macro_line, comment_text)
             macro_line = self._set_special_macro('patch', patchnum, '-p1',
                                                  macro_line)
-            for cmd, args in six.iteritems(cmds):
+            for cmd, args in cmds.items():
                 if cmd in ('if', 'ifarch'):
                     self._content.insert_before(macro_line, '%%%s %s\n' %
                                                 (cmd, args))
@@ -777,11 +782,11 @@ class SpecFile(object):
         """
         orig = None
         sources = self.sources()
-        for num, filename in sorted(six.iteritems(sources)):
+        for num, filename in sorted(sources.items()):
             src = {'num': num, 'filename': os.path.basename(filename),
                    'uri': filename}
             src['filename_base'], src['archive_fmt'], src['compression'] = \
-                parse_archive_filename(os.path.basename(filename))
+                Archive.parse_filename(os.path.basename(filename))
             if (src['filename_base'].startswith(self.name) and
                     src['archive_fmt']):
                 # Take the first archive that starts with pkg name
@@ -851,7 +856,7 @@ def guess_spec_repo(repo, treeish, topdir='', recursive=True, preferred_name=Non
     """
     topdir = topdir.rstrip('/') + ('/') if topdir else ''
     try:
-        file_list = [nam for (mod, typ, sha, nam) in
+        file_list = [nam.decode() for (mod, typ, sha, nam) in
                      repo.list_tree(treeish, recursive, topdir) if typ == 'blob']
     except GitRepositoryError as err:
         raise NoSpecError("Cannot find spec file from treeish %s, Git error: %s"
@@ -863,7 +868,7 @@ def guess_spec_repo(repo, treeish, topdir='', recursive=True, preferred_name=Non
 def spec_from_repo(repo, treeish, spec_path):
     """Get and parse a spec file from a give Git treeish"""
     try:
-        spec = SpecFile(filedata=repo.show('%s:%s' % (treeish, spec_path)))
+        spec = SpecFile(filedata=repo.show('%s:%s' % (treeish, spec_path)).decode())
         spec.specdir = os.path.dirname(spec_path)
         spec.specfile = os.path.basename(spec_path)
         return spec
@@ -977,9 +982,9 @@ def filter_version(evr, *keys):
     @return: new version dict
     @rtype: C{dict} of C{str}
 
-    >>> filter_version({'epoch': 'foo', 'upstreamversion': 'bar', 'vendor': 'baz'}, 'vendor').keys()
+    >>> sorted(list(filter_version({'epoch': 'foo', 'upstreamversion': 'bar', 'vendor': 'baz'}, 'vendor').keys()))
     ['epoch', 'upstreamversion']
-    >>> filter_version({'epoch': 'foo', 'upstreamversion': 'bar', 'revision': 'baz'}, 'epoch', 'revision').keys()
+    >>> list(filter_version({'epoch': 'foo', 'upstreamversion': 'bar', 'revision': 'baz'}, 'epoch', 'revision').keys())
     ['upstreamversion']
     """
     return {k: evr[k] for k in evr if k not in keys}
