@@ -31,92 +31,19 @@ from gbp.config import (GbpOptionParserDebian, GbpOptionGroup)
 from gbp.deb.git import (GitRepositoryError, DebianGitRepository)
 from gbp.deb.source import DebianSource, DebianSourceError, FileVfs
 from gbp.deb.format import DebianSourceFormat
-from gbp.format import format_str
 from gbp.git.vfs import GitVfs
 from gbp.deb.upstreamsource import DebianUpstreamSource
 from gbp.errors import GbpError
-from gbp.scripts.common.pq import is_pq_branch, pq_branch_base
 import gbp.log
 import gbp.notifications
 from gbp.scripts.common.buildpackage import (index_name, wc_name,
-                                             git_archive_submodules,
-                                             git_archive_single, dump_tree,
+                                             dump_tree,
                                              write_wc, drop_index)
 from gbp.scripts.common import ExitCodes
 from gbp.scripts.common.hook import Hook
-from gbp.pkg import compressor_opts, compressor_aliases, parse_archive_filename
 
-
-# upstream tarball preparation
-def git_archive(repo, source, output_dir, treeish, comp_type, comp_level, with_submodules, component=None):
-    "create a compressed orig tarball in output_dir using git_archive"
-    try:
-        comp_opts = compressor_opts[comp_type][0]
-    except KeyError:
-        raise GbpError("Unsupported compression type '%s'" % comp_type)
-
-    output = os.path.join(output_dir, du.orig_file(source, comp_type, component=component))
-    prefix = "%s-%s" % (source.name, source.upstream_version)
-
-    try:
-        if repo.has_submodules() and with_submodules:
-            repo.update_submodules()
-            git_archive_submodules(repo, treeish, output, prefix,
-                                   comp_type, comp_level, comp_opts)
-
-        else:
-            git_archive_single(treeish, output, prefix,
-                               comp_type, comp_level, comp_opts)
-    except (GitRepositoryError, CommandExecFailed):
-        gbp.log.err("Error generating submodules' archives")
-        return False
-    except OSError as e:
-        gbp.log.err("Error creating %s: %s" % (output, str(e)))
-        return False
-    except GbpError:
-        raise
-    except Exception as e:
-        gbp.log.err("Error creating %s: %s" % (output, str(e)))
-        return False
-    return True
-
-
-def prepare_upstream_tarball(repo, source, options, tarball_dir, output_dir):
-    """
-    Make sure we have an upstream tarball. This involves looking in
-    tarball_dir, symlinking or building it.
-    """
-    if not source.is_native() and not source.upstream_version:
-        raise GbpError("Non-native package '%s' "
-                       "has invalid version '%s'" % (source.name, source.version))
-
-    options.comp_type = guess_comp_type(repo,
-                                        options.comp_type,
-                                        source,
-                                        options.tarball_dir)
-
-    orig_files = [du.orig_file(source, options.comp_type)]
-    if options.components:
-        orig_files += [du.orig_file(source, options.comp_type, c) for c in options.components]
-
-    # look in tarball_dir first, if found force a symlink to it
-    if options.tarball_dir:
-        gbp.log.debug("Looking for orig tarballs '%s' at '%s'" % (", ".join(orig_files), tarball_dir))
-        missing = du.DebianPkgPolicy.symlink_origs(orig_files, tarball_dir, output_dir, force=True)
-        if missing:
-            msg = "Tarballs '%s' not found at '%s'" % (", ".join(missing), tarball_dir)
-        else:
-            msg = "All Orig tarballs '%s' found at '%s'" % (", ".join(orig_files), tarball_dir)
-        gbp.log.info(msg)
-    if options.no_create_orig:
-        return
-    # Create tarball if missing or forced
-    if not du.DebianPkgPolicy.has_origs(orig_files, output_dir) or options.force_create:
-        if not pristine_tar_build_orig(repo, source, output_dir, options):
-            upstream_tree = git_archive_build_orig(repo, source, output_dir, options)
-            if options.pristine_tar_commit:
-                pristine_tar_commit(repo, source, options, output_dir, orig_files[0], upstream_tree)
-    pristine_tar_verify_orig(repo, source, options, output_dir, orig_files)
+from gbp.scripts.export_orig import prepare_upstream_tarballs
+from gbp.scripts.tag import perform_tagging
 
 
 def pristine_tar_commit(repo, source, options, output_dir, orig_file, upstream_tree):
@@ -173,8 +100,8 @@ def export_source(repo, tree, source, options, dest_dir, tarball_dir):
         if source.is_native():
             raise GbpError("Cannot overlay Debian native package")
         extract_orig(os.path.join(tarball_dir,
-                                  du.orig_file(source,
-                                               options.comp_type)),
+                                  source.upstream_tarball_name(
+                                      options.comp_type)),
                      dest_dir)
 
     gbp.log.info("Exporting '%s' to '%s'" % (options.export, dest_dir))
@@ -260,176 +187,6 @@ def clean_working_tree(options, repo):
             raise GbpError("Use --git-ignore-new to ignore.")
 
 
-def pristine_tar_prepare_orig_tree(repo, source, options):
-    """Make sure the upstream tree exists
-    In case of component tarballs we need to recreate a tree for the
-    main tarball without the component subdirs.
-    """
-    if options.components:
-        try:
-            upstream_tag = repo.version_to_tag(options.upstream_tag,
-                                               source.upstream_version)
-            tree_name = "%s^{tree}" % upstream_tag
-            repo.tree_drop_dirs(tree_name, options.components)
-        except GitRepositoryError:
-            raise GbpError("Couldn't find upstream tree '%s' to create "
-                           "orig tarball via pristine-tar" % tree_name)
-
-
-def pristine_tar_build_orig(repo, source, output_dir, options):
-    """
-    Build orig tarball using pristine-tar
-
-    @returns: C{True} if tarball was build, C{False} otherwise
-    """
-    if options.pristine_tar:
-        if not repo.has_branch(repo.pristine_tar_branch):
-            gbp.log.warn('Pristine-tar branch "%s" not found' %
-                         repo.pristine_tar.branch)
-
-        pristine_tar_prepare_orig_tree(repo, source, options)
-        try:
-            repo.pristine_tar.checkout(source.name,
-                                       source.upstream_version,
-                                       options.comp_type,
-                                       output_dir)
-            for component in options.components:
-                repo.pristine_tar.checkout(source.name,
-                                           source.upstream_version,
-                                           options.comp_type,
-                                           output_dir,
-                                           component=component)
-            return True
-        except CommandExecFailed:
-            if options.pristine_tar_commit:
-                gbp.log.debug("pristine-tar checkout failed, "
-                              "will commit tarball due to "
-                              "'--pristine-tar-commit'")
-            else:
-                raise
-    return False
-
-
-def pristine_tar_verify_orig(repo, source, options, output_dir, orig_files):
-    """
-    Verify orig tarballs using prstine tar
-
-    @returns: C{True} if tarball was build, C{False} otherwise
-    """
-    if not options.pristine_tar:
-        return True
-
-    pristine_tar_prepare_orig_tree(repo, source, options)
-    for f in orig_files:
-        repo.pristine_tar.verify(os.path.join(output_dir, f))
-    return True
-
-
-def get_upstream_tree(repo, source, options):
-    """Determine the upstream tree from the given options"""
-    if options.upstream_tree.upper() == 'TAG':
-        if source.upstream_version is None:
-            raise GitRepositoryError("Can't determine upstream version from changelog")
-        upstream_tree = repo.version_to_tag(options.upstream_tag,
-                                            source.upstream_version)
-    elif options.upstream_tree.upper() == 'BRANCH':
-        if not repo.has_branch(options.upstream_branch):
-            raise GbpError("%s is not a valid branch" % options.upstream_branch)
-        upstream_tree = options.upstream_branch
-    elif options.upstream_tree.upper() == 'SLOPPY':
-        tree_name = "%s^{tree}" % options.debian_branch
-        upstream_tree = repo.tree_drop_dirs(tree_name, ["debian"])
-    else:
-        upstream_tree = options.upstream_tree
-    if not repo.has_treeish(upstream_tree):
-        raise GbpError("%s is not a valid treeish" % upstream_tree)
-    return upstream_tree
-
-
-def git_archive_build_orig(repo, source, output_dir, options):
-    """
-    Build orig tarball(s) using git-archive
-
-    @param source: the changelog of the package we're acting on
-    @type source: L{DebianSource}
-    @param output_dir: where to put the tarball
-    @type output_dir: C{Str}
-    @param options: the parsed options
-    @type options: C{dict} of options
-    @return: the tree we built the tarball from
-    @rtype: C{str}
-    """
-    upstream_tree = get_upstream_tree(repo, source, options)
-    gbp.log.info("Creating %s from '%s'" % (du.orig_file(source,
-                                                         options.comp_type),
-                                            upstream_tree))
-    comp_level = int(options.comp_level) if options.comp_level != '' else None
-    gbp.log.debug("Building upstream tarball with compression '%s'%s" %
-                  (options.comp_type,
-                   "' -%s'" % comp_level if comp_level is not None else ''))
-    main_tree = repo.tree_drop_dirs(upstream_tree, options.components) if options.components else upstream_tree
-    if not git_archive(repo, source, output_dir, main_tree,
-                       options.comp_type,
-                       comp_level,
-                       options.with_submodules):
-        raise GbpError("Cannot create upstream tarball at '%s'" % output_dir)
-    for component in options.components:
-        subtree = repo.tree_get_dir(upstream_tree, component)
-        if not subtree:
-            raise GbpError("No tree for '%s' found in '%s' to create additional tarball from"
-                           % (component, upstream_tree))
-        gbp.log.info("Creating additional tarball '%s' from '%s'"
-                     % (du.orig_file(source, options.comp_type, component=component),
-                        subtree))
-        if not git_archive(repo, source, output_dir, subtree,
-                           options.comp_type,
-                           comp_level,
-                           options.with_submodules,
-                           component=component):
-            raise GbpError("Cannot create additional tarball %s at '%s'"
-                           % (component, output_dir))
-    return upstream_tree
-
-
-def guess_comp_type(repo, comp_type, source, tarball_dir):
-    """Guess compression type"""
-    if comp_type != 'auto':
-        comp_type = compressor_aliases.get(comp_type, comp_type)
-        if comp_type not in compressor_opts:
-            gbp.log.warn("Unknown compression type - guessing.")
-            comp_type = 'auto'
-
-    if comp_type == 'auto':
-        if not repo.has_pristine_tar_branch():
-            if not tarball_dir:
-                tarball_dir = '..'
-            detected = None
-            for comp in compressor_opts.keys():
-                if du.DebianPkgPolicy.has_orig(du.orig_file(source, comp), tarball_dir):
-                    if detected is not None:
-                        raise GbpError("Multiple orig tarballs found.")
-                    detected = comp
-            if detected is not None:
-                comp_type = detected
-            else:
-                comp_type = 'gzip'
-        else:
-            regex = 'pristine-tar .* %s_%s\.orig.tar\.' % (source.name, source.upstream_version)
-            commits = repo.grep_log(regex, repo.pristine_tar_branch)
-            if commits:
-                commit = commits[-1]
-                gbp.log.debug("Found pristine-tar commit at '%s'" % commit)
-            else:
-                commit = repo.pristine_tar_branch
-            tarball = repo.get_commit_info(commit)['subject']
-            (base_name, archive_fmt, comp_type) = parse_archive_filename(tarball)
-            gbp.log.debug("Determined compression type '%s'" % comp_type)
-            if not comp_type:
-                comp_type = 'gzip'
-                gbp.log.warn("Unknown compression type of %s, assuming %s" % (tarball, comp_type))
-    return comp_type
-
-
 def check_tag(options, repo, source):
     """Perform specified consistency checks on git history"""
     tag = repo.version_to_tag(options.debian_tag, source.version)
@@ -442,24 +199,29 @@ def get_pbuilder_dist(options, repo, native=False):
     """
     Determin the dist to build for with pbuilder/cowbuilder
     """
+    dist = None
     if options.pbuilder_dist == 'DEP14':
+        vendor = du.get_vendor().lower()
         branch = repo.branch
         if not branch:
             raise GbpError("Failed to setup DIST for %s. "
                            "Can't determine current branch." % options.builder)
-        else:
-            parts = branch.rsplit('/')
-            if len(parts) == 2:
-                vendor = du.get_vendor().lower()
-                suite = parts[1]
-                if vendor == parts[0]:
-                    dist = '' if suite in ['sid', 'master'] else suite
-                else:
-                    dist = '%s_%s' % (parts[0], suite)
-            elif len(parts) == 1 and native and branch in ['master', 'sid']:
-                dist = ''
+        parts = branch.rsplit('/')
+        if len(parts) == 2:  # e.g. debian/stretch
+            suite = parts[1]
+            if vendor == parts[0]:
+                dist = '' if suite in ['sid', 'master'] else suite
             else:
-                raise GbpError("DEP14 DIST: Current branch '%s' does not match vendor/suite" % branch)
+                dist = '%s_%s' % (parts[0], suite)
+        # Branches in Debian often omit the debian/ prefix
+        elif len(parts) == 1 and vendor.lower() == "debian":
+            if branch in ['master', 'sid']:
+                dist = ''
+            elif branch in du.Releases:
+                dist = branch
+
+        if dist is None:
+            raise GbpError("DEP14 DIST: Current branch '%s' does not match vendor/suite" % branch)
     else:
         dist = options.pbuilder_dist
     return dist
@@ -540,6 +302,35 @@ def changes_file_suffix(dpkg_args):
         return os.getenv('ARCH', None) or du.get_arch()
 
 
+def changes_file_name(source, build_dir, dpkg_args):
+    return os.path.abspath("%s/../%s_%s_%s.changes" %
+                           (build_dir,
+                            source.changelog.name,
+                            source.changelog.noepoch,
+                            changes_file_suffix(dpkg_args)))
+
+
+def check_branch(repo, options):
+    """
+    Check if we're on the right branch and bail out otherwise
+
+    returns: the current branch or C{None} if in detached head mode
+    """
+    branch = None
+    try:
+        branch = repo.get_branch()
+    except GitRepositoryError:
+        # Not being on any branch is o.k. with --git-ignore-branch
+        if not options.ignore_branch:
+            raise
+
+    ignore = options.ignore_new or options.ignore_branch
+    if branch != options.debian_branch and not ignore:
+            gbp.log.err("You are not on branch '%s' but on '%s'" % (options.debian_branch, branch))
+            raise GbpError("Use --git-ignore-branch to ignore or --git-debian-branch to set the branch name.")
+    return branch
+
+
 def build_parser(name, prefix=None):
     try:
         parser = GbpOptionParserDebian(command=os.path.basename(name), prefix=prefix)
@@ -636,8 +427,6 @@ def build_parser(name, prefix=None):
                                         help="export treeish object TREEISH, "
                                              "default is '%(export)s'", metavar="TREEISH")
     export_group.add_boolean_config_file_option(option_name="purge", dest="purge")
-    export_group.add_option("--git-dont-purge", action="store_true", dest="dont_purge", default=False,
-                            help="deprecated, use --git-no-purge instead")
     export_group.add_boolean_config_file_option(option_name="overlay", dest="overlay")
     return parser
 
@@ -669,11 +458,6 @@ def parse_args(argv, prefix):
         gbp.log.err("Overlay must be used with --git-export-dir")
         return None, None, None
 
-    # --git-dont-purge is deprecated:
-    if options.dont_purge:
-        gbp.log.warn("--git-dont-purge is depreceted, use --git-no-purge instead")
-        options.purge = False
-
     if options.components and options.pristine_tar_commit:
         gbp.log.warn("Components specified, pristine-tar-commit not yet supported - disabling it.")
         options.pristine_tar_commit = False
@@ -686,7 +470,6 @@ def main(argv):
     retval = 0
     prefix = "git-"
     source = None
-    branch = None
     hook_env = {}
 
     options, gbp_args, dpkg_args = parse_args(argv, prefix)
@@ -702,20 +485,7 @@ def main(argv):
 
     try:
         clean_working_tree(options, repo)
-
-        try:
-            branch = repo.get_branch()
-        except GitRepositoryError:
-            # Not being on any branch is o.k. with --git-ignore-branch
-            if not options.ignore_branch:
-                raise
-
-        if not options.ignore_new and not options.ignore_branch:
-            if branch != options.debian_branch:
-                gbp.log.err("You are not on branch '%s' but on '%s'" % (options.debian_branch, branch))
-                raise GbpError("Use --git-ignore-branch to ignore or --git-debian-branch to set the branch name.")
-
-        head = repo.head
+        check_branch(repo, options)
         tree = maybe_write_tree(repo, options)
         source = source_vfs(repo, options, tree)
 
@@ -724,6 +494,13 @@ def main(argv):
         if not options.tag_only:
             output_dir = prepare_output_dir(options.export_dir)
             tarball_dir = options.tarball_dir or output_dir
+            tmp_dir = os.path.join(output_dir, "%s-tmp" % source.sourcepkg)
+            build_env, hook_env = setup_pbuilder(options, repo, source.is_native())
+            major = (source.debian_version if source.is_native()
+                     else source.upstream_version)
+            export_dir = os.path.join(output_dir, "%s-%s" % (source.sourcepkg, major))
+            build_dir = export_dir if options.export_dir else repo.path
+            changes_file = changes_file_name(source, build_dir, dpkg_args)
 
             # Get/build the upstream tarball if necessary. We delay this in
             # case of a postexport hook so the hook gets a chance to modify the
@@ -734,15 +511,12 @@ def main(argv):
                 if options.postexport:
                     gbp.log.info("Postexport hook set, delaying tarball creation")
                 else:
-                    prepare_upstream_tarball(repo, source, options, tarball_dir,
-                                             output_dir)
-
-            build_env, hook_env = setup_pbuilder(options, repo, source.is_native())
+                    prepare_upstream_tarballs(repo, source, options, tarball_dir,
+                                              output_dir)
 
             # Export to another build dir if requested:
             if options.export_dir:
-                tmp_dir = os.path.join(output_dir, "%s-tmp" % source.sourcepkg)
-                export_source(repo, tree, source, options, tmp_dir, output_dir)
+                export_source(repo, tree, source, options, tmp_dir, tarball_dir)
 
                 # Run postexport hook
                 if options.postexport:
@@ -752,21 +526,14 @@ def main(argv):
                                             'GBP_TMP_DIR': tmp_dir})
                          )(dir=tmp_dir)
 
-                major = (source.debian_version if source.is_native()
-                         else source.upstream_version)
-                export_dir = os.path.join(output_dir, "%s-%s" % (source.sourcepkg, major))
                 gbp.log.info("Moving '%s' to '%s'" % (tmp_dir, export_dir))
                 move_old_export(export_dir)
                 os.rename(tmp_dir, export_dir)
 
                 # Delayed tarball creation in case a postexport hook is used:
                 if not source.is_native() and options.postexport:
-                    prepare_upstream_tarball(repo, source, options, tarball_dir,
-                                             output_dir)
-                build_dir = export_dir
-            else:
-                build_dir = repo.path
-
+                    prepare_upstream_tarballs(repo, source, options, tarball_dir,
+                                              output_dir)
             if options.prebuild:
                 Hook('Prebuild', options.prebuild,
                      extra_env=Hook.md(hook_env,
@@ -782,44 +549,15 @@ def main(argv):
                                            {'GBP_BUILD_DIR': build_dir})
                          )(dir=build_dir)
             if options.postbuild:
-                changes = os.path.abspath("%s/../%s_%s_%s.changes" %
-                                          (build_dir,
-                                           source.changelog.name,
-                                           source.changelog.noepoch,
-                                           changes_file_suffix(dpkg_args)))
-                gbp.log.debug("Looking for changes file %s" % changes)
+                gbp.log.debug("Looking for changes file %s" % changes_file)
                 Hook('Postbuild', options.postbuild,
                      extra_env=Hook.md(hook_env,
-                                       {'GBP_CHANGES_FILE': changes,
+                                       {'GBP_CHANGES_FILE': changes_file,
                                         'GBP_BUILD_DIR': build_dir})
                      )()
         if options.tag or options.tag_only:
-            if branch and is_pq_branch(branch):
-                commit = repo.get_merge_base(branch, pq_branch_base(branch))
-            else:
-                commit = head
+            perform_tagging(repo, source, options, hook_env)
 
-            tag = repo.version_to_tag(options.debian_tag, source.version)
-            gbp.log.info("Tagging %s as %s" % (source.version, tag))
-            if options.retag and repo.has_tag(tag):
-                repo.delete_tag(tag)
-            tag_msg = format_str(options.debian_tag_msg,
-                                 dict(pkg=source.sourcepkg,
-                                      version=source.version))
-            repo.create_tag(name=tag,
-                            msg=tag_msg,
-                            sign=options.sign_tags,
-                            commit=commit,
-                            keyid=options.keyid)
-
-            if options.posttag:
-                sha = repo.rev_parse("%s^{}" % tag)
-                Hook('Posttag', options.posttag,
-                     extra_env=Hook.md(hook_env,
-                                       {'GBP_TAG': tag,
-                                        'GBP_BRANCH': branch or '(no branch)',
-                                        'GBP_SHA1': sha})
-                     )()
     except KeyboardInterrupt:
         retval = 1
         gbp.log.err("Interrupted. Aborting.")
