@@ -1,6 +1,6 @@
 # vim: set fileencoding=utf-8 :
 #
-# (C) 2011,2015 Guido Günther <agx@sigxcpu.org>
+# (C) 2011,2015,2017 Guido Günther <agx@sigxcpu.org>
 # (C) 2012 Intel Corporation <markus.lehtonen@linux.intel.com>
 #    This program is free software; you can redistribute it and/or modify
 #    it under the terms of the GNU General Public License as published by
@@ -20,7 +20,6 @@
 
 import re
 import os
-import subprocess
 import datetime
 import time
 from email.message import Message
@@ -52,23 +51,29 @@ def pq_branch_name(branch):
     get the patch queue branch corresponding to branch
 
     >>> pq_branch_name("patch-queue/master")
+    'patch-queue/master'
     >>> pq_branch_name("foo")
     'patch-queue/foo'
     """
     if not is_pq_branch(branch):
         return PQ_BRANCH_PREFIX + branch
+    else:
+        return branch
 
 
-def pq_branch_base(pq_branch):
+def pq_branch_base(branch):
     """
     get the branch corresponding to the given patch queue branch
 
     >>> pq_branch_base("patch-queue/master")
     'master'
     >>> pq_branch_base("foo")
+    'foo'
     """
-    if is_pq_branch(pq_branch):
-        return pq_branch[len(PQ_BRANCH_PREFIX):]
+    if is_pq_branch(branch):
+        return branch[len(PQ_BRANCH_PREFIX):]
+    else:
+        return branch
 
 
 def parse_gbp_commands(info, cmd_tag, noarg_cmds, arg_cmds, filter_cmds=None):
@@ -136,7 +141,7 @@ def write_patch_file(filename, commit_info, diff):
         gbp.log.debug("I won't generate empty diff %s" % filename)
         return None
     try:
-        with open(filename, 'w') as patch:
+        with open(filename, 'wb') as patch:
             msg = Message()
             charset = Charset('utf-8')
             charset.body_encoding = None
@@ -148,26 +153,34 @@ def write_patch_file(filename, commit_info, diff):
             # Git compat: put name in quotes if special characters found
             if re.search("[,.@()\[\]\\\:;]", name):
                 name = '"%s"' % name
-            from_header = Header(unicode(name, 'utf-8'), charset, 77, 'from')
-            from_header.append(unicode('<%s>' % email))
+            from_header = Header(header_name='from')
+            try:
+                from_header.append(name, 'us-ascii')
+            except UnicodeDecodeError:
+                from_header.append(name, charset)
+            from_header.append('<%s>' % email)
             msg['From'] = from_header
             date = commit_info['author'].datetime
             datestr = date.strftime('%a, %-d %b %Y %H:%M:%S %z')
-            msg['Date'] = Header(unicode(datestr, 'utf-8'), charset, 77, 'date')
-            msg['Subject'] = Header(unicode(commit_info['subject'], 'utf-8'),
-                                    charset, 77, 'subject')
+            msg['Date'] = Header(datestr, 'us-ascii', 'date')
+            subject_header = Header(header_name='subject')
+            try:
+                subject_header.append(commit_info['subject'], 'us-ascii')
+            except UnicodeDecodeError:
+                subject_header.append(commit_info['subject'], charset)
+            msg['Subject'] = subject_header
             # Write message body
             if commit_info['body']:
                 # Strip extra linefeeds
                 body = commit_info['body'].rstrip() + '\n'
                 try:
-                    msg.set_payload(body.encode('ascii'))
-                except UnicodeDecodeError:
+                    msg.set_payload(body.encode('us-ascii'))
+                except (UnicodeEncodeError):
                     msg.set_payload(body, charset)
-            patch.write(msg.as_string(unixfrom=False))
+            patch.write(msg.as_string(unixfrom=False, maxheaderlen=77).encode('utf-8'))
 
             # Write diff
-            patch.write('---\n')
+            patch.write(b'---\n')
             patch.write(diff)
     except IOError as err:
         raise GbpError('Unable to create patch file: %s' % err)
@@ -177,7 +190,7 @@ def write_patch_file(filename, commit_info, diff):
 DEFAULT_PATCH_NUM_PREFIX_FORMAT = "%04d-"
 
 
-def format_patch(outdir, repo, commit_info, series, numbered=True,
+def format_patch(outdir, repo, commit_info, series, abbrev, numbered=True,
                  path_exclude_regex=None, topic='', name=None, renumber=False,
                  patch_num_prefix_format=DEFAULT_PATCH_NUM_PREFIX_FORMAT):
     """Create patch of a single commit"""
@@ -227,14 +240,14 @@ def format_patch(outdir, repo, commit_info, series, numbered=True,
     patch = None
     if paths:
         diff = repo.diff('%s^!' % commit_info['id'], paths=paths, stat=80,
-                         summary=True, text=True, abbrev=7, renames=False)
+                         summary=True, text=True, abbrev=abbrev, renames=False)
         patch = write_patch_file(filepath, commit_info, diff)
         if patch:
             series.append(patch)
     return patch
 
 
-def format_diff(outdir, filename, repo, start, end, path_exclude_regex=None):
+def format_diff(outdir, filename, repo, start, end, abbrev, path_exclude_regex=None):
     """Create a patch of diff between two repository objects"""
 
     info = {'author': repo.get_author_info()}
@@ -252,7 +265,7 @@ def format_diff(outdir, filename, repo, start, end, path_exclude_regex=None):
     paths = patch_path_filter(file_status, path_exclude_regex)
     if paths:
         diff = repo.diff(start, end, paths=paths, stat=80, summary=True,
-                         text=True, abbrev=7, renames=False)
+                         text=True, abbrev=abbrev, renames=False)
         return write_patch_file(filename, info, diff)
     return None
 
@@ -261,22 +274,18 @@ def get_maintainer_from_control(repo):
     """Get the maintainer from the control file"""
     control = os.path.join(repo.path, 'debian', 'control')
 
-    cmd = 'sed -n -e \"s/Maintainer: \\+\\(.*\\)/\\1/p\" %s' % control
-    cmdout = subprocess.Popen(cmd, shell=True,
-                              stdout=subprocess.PIPE).stdout.readlines()
-
-    if len(cmdout) > 0:
-        maintainer = cmdout[0].strip()
-        m = re.match('(?P<name>.*[^ ]) *<(?P<email>.*)>', maintainer)
-        if m:
-            return GitModifier(m.group('name'), m.group('email'))
-
+    maint_re = re.compile('Maintainer: +(?P<name>.*[^ ]) *<(?P<email>.*)>')
+    with open(control, encoding='utf-8') as f:
+        for line in f:
+            m = maint_re.match(line)
+            if m:
+                return GitModifier(m.group('name'), m.group('email'))
     return GitModifier()
 
 
 def switch_to_pq_branch(repo, branch):
     """
-    Switch to patch-queue branch if not already there, create it if it
+    Switch to patch-queue branch if not already on it.
     doesn't exist yet
     """
     if is_pq_branch(branch):
@@ -284,11 +293,8 @@ def switch_to_pq_branch(repo, branch):
 
     pq_branch = pq_branch_name(branch)
     if not repo.has_branch(pq_branch):
-        try:
-            repo.create_branch(pq_branch)
-        except GitRepositoryError:
-            raise GbpError("Cannot create patch-queue branch '%s'. "
-                           "Try 'rebase' instead." % pq_branch)
+        raise GbpError("Branch '%s' does not exist, try "
+                       "'import' instead" % pq_branch)
 
     gbp.log.info("Switching to '%s'" % pq_branch)
     repo.set_branch(pq_branch)
@@ -332,24 +338,10 @@ def apply_and_commit_patch(repo, patch, fallback_author, topic=None, name=None):
 
 
 def drop_pq(repo, branch):
-    if is_pq_branch(branch):
-        gbp.log.err("On a patch-queue branch, can't drop it.")
-        raise GbpError
-    else:
-        pq_branch = pq_branch_name(branch)
-
+    repo.checkout(pq_branch_base(branch))
+    pq_branch = pq_branch_name(branch)
     if repo.has_branch(pq_branch):
         repo.delete_branch(pq_branch)
         gbp.log.info("Dropped branch '%s'." % pq_branch)
     else:
         gbp.log.info("No patch queue branch found - doing nothing.")
-
-
-def switch_pq(repo, current):
-    """Switch to patch-queue branch if on base branch and vice versa"""
-    if is_pq_branch(current):
-        base = pq_branch_base(current)
-        gbp.log.info("Switching to %s" % base)
-        repo.checkout(base)
-    else:
-        switch_to_pq_branch(repo, current)
