@@ -1,6 +1,6 @@
 # vim: set fileencoding=utf-8 :
 #
-# (C) 2006, 2007, 2011, 2012, 2015, 2016 Guido Guenther <agx@sigxcpu.org>
+# (C) 2006, 2007, 2011, 2012, 2015-2017 Guido Guenther <agx@sigxcpu.org>
 #    This program is free software; you can redistribute it and/or modify
 #    it under the terms of the GNU General Public License as published by
 #    the Free Software Foundation; either version 2 of the License, or
@@ -80,12 +80,9 @@ def apply_patch(diff):
     try:
         ret = pipe.copy('', '')
         if ret:
-            gbp.log.err("Error import %s: %d" % (diff, ret))
-            return False
+            raise GbpError("Error import %s: %d" % (diff, ret))
     except OSError as err:
-        gbp.log.err("Error importing %s: %s" % (diff, err[0]))
-        return False
-    return True
+        raise GbpError("Error importing %s: %s" % (diff, err[0]))
 
 
 def apply_deb_tgz(deb_tgz):
@@ -94,7 +91,20 @@ def apply_deb_tgz(deb_tgz):
     if os.path.isdir('debian'):
         shutil.rmtree('debian')
     gbpc.UnpackTarArchive(deb_tgz, ".")()
-    return True
+
+
+def get_changes(dir, repo, debian_branch):
+    if repo.empty:
+        version = "0~"
+    else:
+        vfs = GitVfs(repo, debian_branch)
+        try:
+            with vfs.open('debian/changelog') as f:
+                version = ChangeLog(contents=f.read()).version
+        except IOError:
+            version = "0~"  # Use full history if debian branch has no changelog
+    cl = ChangeLog(filename=os.path.join(dir, 'debian/changelog'))
+    return cl.get_changes(version)
 
 
 def get_author_from_changelog(dir):
@@ -143,53 +153,126 @@ def check_parents(repo, branch, tag):
     return parents
 
 
-def apply_debian_patch(repo, unpack_dir, src, options, tag, is_empty):
+def apply_debian_patch(repo, source, dsc, upstream_commit, options):
     """apply the debian patch and tag appropriately"""
+
     try:
-        os.chdir(unpack_dir)
+        os.chdir(source.unpacked)
 
-        if src.diff and not apply_patch(src.diff):
-            raise GbpError
-
-        if src.deb_tgz and not apply_deb_tgz(src.deb_tgz):
-            raise GbpError
+        if dsc.diff:
+            apply_patch(dsc.diff)
+        elif dsc.deb_tgz:
+            apply_deb_tgz(dsc.deb_tgz)
+        else:
+            raise GbpError("Neither a Debian diff nor tarball found")
 
         if os.path.exists('debian/rules'):
             os.chmod('debian/rules', 0o755)
         os.chdir(repo.path)
 
-        parents = check_parents(repo,
-                                options.debian_branch,
-                                tag)
-
-        author = get_author_from_changelog(unpack_dir)
+        parents = check_parents(repo, options.debian_branch, upstream_commit)
+        author = get_author_from_changelog(source.unpacked)
         committer = get_committer_from_author(author, options)
 
-        changes = get_changes(unpack_dir,
+        changes = get_changes(source.unpacked,
                               repo,
-                              is_empty,
                               options.debian_branch)
-        commit_msg = "Import Debian changes %s\n%s" % (src.version, changes)
-        commit = repo.commit_dir(unpack_dir,
+        commit_msg = "Import Debian changes %s\n%s" % (dsc.version, changes)
+        commit = repo.commit_dir(source.unpacked,
                                  commit_msg,
                                  branch=options.debian_branch,
                                  other_parents=parents,
                                  author=author,
                                  committer=committer)
         if not options.skip_debian_tag:
-            repo.create_tag(repo.version_to_tag(options.debian_tag, src.version),
-                            msg="Debian release %s" % src.version,
+            repo.create_tag(repo.version_to_tag(options.debian_tag, dsc.version),
+                            msg="Debian release %s" % dsc.version,
                             commit=commit,
                             sign=options.sign_tags,
                             keyid=options.keyid)
     except (gbpc.CommandExecFailed, GitRepositoryError) as err:
         msg = str(err) or 'Unknown error, please report a bug'
-        gbp.log.err("Failed to import Debian package: %s" % msg)
-        raise GbpError
+        raise GbpError("Failed to import Debian package: %s" % msg)
     finally:
         os.chdir(repo.path)
 
 
+def create_missing_branch(repo, branch, options):
+    if not repo.has_branch(branch):
+        if options.create_missing_branches:
+            gbp.log.info("Creating missing branch '%s'" % branch)
+            repo.create_branch(branch)
+        else:
+            raise GbpError(no_upstream_branch_msg % branch +
+                           "\nAlso check the --create-missing-branches option.")
+
+
+def import_native(repo, source, dsc, options):
+    tag = repo.version_to_tag(options.debian_tag, dsc.upstream_version)
+    msg = "Debian version %s" % dsc.upstream_version
+
+    gbp.log.info("Tag %s not found, importing Debian tarball" % tag)
+    if repo.empty:
+        branch = None
+    else:
+        branch = options.debian_branch
+        create_missing_branch(repo, branch, options)
+
+    author = get_author_from_changelog(source.unpacked)
+    committer = get_committer_from_author(author, options)
+    commit_msg = "Import %s\n%s" % (msg, get_changes(source.unpacked,
+                                                     repo,
+                                                     options.debian_branch))
+    commit = repo.commit_dir(source.unpacked,
+                             commit_msg,
+                             branch,
+                             author=author,
+                             committer=committer)
+    if not options.skip_debian_tag:
+        repo.create_tag(name=tag,
+                        msg=msg,
+                        commit=commit,
+                        sign=options.sign_tags,
+                        keyid=options.keyid)
+    if repo.empty and not repo.has_branch(options.debian_branch):
+        repo.create_branch(options.debian_branch, commit)
+    return commit
+
+
+def import_upstream(repo, source, dsc, options):
+    tag = repo.version_to_tag(options.upstream_tag, dsc.upstream_version)
+    msg = "Upstream version %s" % dsc.upstream_version
+
+    if repo.empty:
+        branch = None
+    else:
+        branch = options.upstream_branch
+        create_missing_branch(repo, branch, options)
+
+    author = committer = {}
+    commit_msg = "Import %s" % msg
+
+    commit = repo.commit_dir(source.unpacked,
+                             commit_msg,
+                             branch,
+                             author=author,
+                             committer=committer)
+
+    # if the repo was just created make sure debian branch is in .git/HEAD
+    # and upstream points to the first commit
+    if repo.empty:
+        if repo.branch != options.debian_branch:
+            repo.rename_branch(repo.branch, options.debian_branch)
+        repo.create_branch(options.upstream_branch, options.debian_branch)
+
+    repo.create_tag(name=tag,
+                    msg=msg,
+                    commit=commit,
+                    sign=options.sign_tags,
+                    keyid=options.keyid)
+    return commit
+
+
 def print_dsc(dsc):
     if dsc.native:
         gbp.log.debug("Debian Native Package")
@@ -271,7 +354,7 @@ def build_parser(name):
                                                 dest="pristine_tar")
     import_group.add_option("--allow-same-version", action="store_true",
                             dest="allow_same_version", default=False,
-                            help="allow to import already imported version")
+                            help="allow import of already imported version")
     import_group.add_boolean_config_file_option(option_name="author-is-committer",
                                                 dest="author_committer")
     import_group.add_boolean_config_file_option(option_name="author-date-is-committer-date",
@@ -347,24 +430,10 @@ def parse_all(argv):
     return options, pkg, target
 
 
-def get_changes(dir, repo, is_empty, debian_branch):
-    if is_empty:
-        version = "0~"
-    else:
-        vfs = GitVfs(repo, debian_branch)
-        try:
-            with vfs.open('debian/changelog') as f:
-                version = ChangeLog(contents=f.read()).version
-        except IOError:
-            version = "0~"  # Use full history if debian branch has no changelog
-    cl = ChangeLog(filename=os.path.join(dir, 'debian/changelog'))
-    return cl.get_changes(version)
-
-
 def main(argv):
     dirs = dict(top=os.path.abspath(os.curdir))
     needs_repo = False
-    ret = 0
+    ret = 1
     skipped = False
 
     options, pkg, target = parse_all(argv)
@@ -374,135 +443,103 @@ def main(argv):
     try:
         try:
             repo = DebianGitRepository('.')
-            is_empty = repo.is_empty()
+            # remember if the was repo initially empty
+            repo.empty = repo.is_empty()
 
             (clean, out) = repo.is_clean()
-            if not clean and not is_empty:
-                gbp.log.err("Repository has uncommitted changes, commit these first: ")
-                raise GbpError(out)
+            if not clean and not repo.empty:
+                raise GbpError("Repository has uncommitted changes, commit these first: %s" % out)
         except GitRepositoryError:
             # no repo found, create one
             needs_repo = True
-            is_empty = True
 
         if options.download:
-            dsc = download_source(pkg,
-                                  dirs=dirs,
-                                  unauth=options.allow_unauthenticated)
+            dscfile = download_source(pkg,
+                                      dirs=dirs,
+                                      unauth=options.allow_unauthenticated)
         else:
-            dsc = pkg
+            dscfile = pkg
 
-        src = DscFile.parse(dsc)
-        if src.pkgformat not in ['1.0', '3.0']:
-            raise GbpError("Importing %s source format not yet supported." % src.pkgformat)
+        dsc = DscFile.parse(dscfile)
+        if dsc.pkgformat not in ['1.0', '3.0']:
+            raise GbpError("Importing %s source format not yet supported." % dsc.pkgformat)
         if options.verbose:
-            print_dsc(src)
+            print_dsc(dsc)
 
         if needs_repo:
-            target = target or src.pkg
+            target = target or dsc.pkg
             if os.path.exists(target):
                 raise GbpError("Directory '%s' already exists. If you want to import into it, "
                                "please change into this directory otherwise move it away first."
                                % target)
             gbp.log.info("No git repository found, creating one.")
             repo = DebianGitRepository.create(target)
+            repo.empty = True
             os.chdir(repo.path)
             repo_setup.set_user_name_and_email(options.repo_user, options.repo_email, repo)
 
         if repo.bare:
             disable_pristine_tar(options, "Bare repository")
 
+        # unpack
         dirs['tmp'] = os.path.abspath(tempfile.mkdtemp(dir='..'))
-        upstream = DebianUpstreamSource(src.tgz)
-        upstream.unpack(dirs['tmp'], options.filters)
-        for (component, tarball) in src.additional_tarballs.items():
+        source = DebianUpstreamSource(dsc.tgz)
+        source.unpack(dirs['tmp'], options.filters)
+        for (component, tarball) in dsc.additional_tarballs.items():
             gbp.log.info("Found component tarball '%s'" % os.path.basename(tarball))
-            unpack_component_tarball(upstream.unpacked, component, tarball, options.filters)
+            unpack_component_tarball(source.unpacked, component, tarball, options.filters)
 
-        format = [(options.upstream_tag, "Upstream"), (options.debian_tag, "Debian")][src.native]
-        tag = repo.version_to_tag(format[0], src.upstream_version)
-        msg = "%s version %s" % (format[1], src.upstream_version)
-
-        if repo.find_version(options.debian_tag, src.version):
-            gbp.log.warn("Version %s already imported." % src.version)
+        if repo.find_version(options.debian_tag, dsc.version):
+            gbp.log.warn("Version %s already imported." % dsc.version)
             if options.allow_same_version:
-                gbp.log.info("Moving tag of version '%s' since import forced" % src.version)
-                move_tag_stamp(repo, options.debian_tag, src.version)
+                gbp.log.info("Moving tag of version '%s' since import forced" % dsc.version)
+                move_tag_stamp(repo, options.debian_tag, dsc.version)
             else:
                 raise SkipImport
 
-        if not repo.find_version(format[0], src.upstream_version):
-            gbp.log.info("Tag %s not found, importing %s tarball" % (tag, format[1]))
-            if is_empty:
-                branch = None
-            else:
-                branch = [options.upstream_branch,
-                          options.debian_branch][src.native]
-                if not repo.has_branch(branch):
-                    if options.create_missing_branches:
-                        gbp.log.info("Creating missing branch '%s'" % branch)
-                        repo.create_branch(branch)
-                    else:
-                        gbp.log.err(no_upstream_branch_msg % branch +
-                                    "\nAlso check the --create-missing-branches option.")
-                        raise GbpError
+        # import
+        if dsc.native:
+            import_native(repo, source, dsc, options)
+        else:
+            imported = False
+            commit = repo.find_version(options.upstream_tag, dsc.upstream_version)
+            if not repo.find_version(options.upstream_tag, dsc.upstream_version):
+                commit = import_upstream(repo, source, dsc, options)
+                imported = True
 
-            if src.native:
-                author = get_author_from_changelog(upstream.unpacked)
-                committer = get_committer_from_author(author, options)
-                commit_msg = "Import %s\n%s" % (msg, get_changes(upstream.unpacked,
-                                                                 repo,
-                                                                 is_empty,
-                                                                 options.debian_branch))
-            else:
-                author = committer = {}
-                commit_msg = "Import %s" % msg
+            if not repo.has_branch(options.debian_branch):
+                if options.create_missing_branches:
+                    repo.create_branch(options.debian_branch, commit)
+                else:
+                    raise GbpError("Branch %s does not exist, use --create-missing-branches" %
+                                   options.debian_branch)
 
-            commit = repo.commit_dir(upstream.unpacked,
-                                     commit_msg,
-                                     branch,
-                                     author=author,
-                                     committer=committer)
-
-            if not (src.native and options.skip_debian_tag):
-                repo.create_tag(name=tag,
-                                msg=msg,
-                                commit=commit,
-                                sign=options.sign_tags,
-                                keyid=options.keyid)
-            if not src.native:
-                if is_empty:
-                    repo.create_branch(options.upstream_branch, commit)
-            if (not repo.has_branch(options.debian_branch) and
-                    (is_empty or options.create_missing_branches)):
-                repo.create_branch(options.debian_branch, commit)
-        if not src.native:
-            if src.diff or src.deb_tgz:
-                apply_debian_patch(repo, upstream.unpacked, src, options,
-                                   tag, is_empty)
+            if dsc.diff or dsc.deb_tgz:
+                apply_debian_patch(repo, source, dsc, commit, options)
             else:
                 gbp.log.warn("Didn't find a diff to apply.")
-        if repo.get_branch() == options.debian_branch or is_empty:
+
+            if imported and options.pristine_tar:
+                repo.create_pristine_tar_commits(commit,
+                                                 dsc.tgz,
+                                                 dsc.additional_tarballs.items())
+
+        if repo.get_branch() == options.debian_branch or repo.empty:
             # Update HEAD if we modified the checked out branch
             repo.force_head(options.debian_branch, hard=True)
-        if options.pristine_tar and not src.native:
-            repo.create_pristinetar_commits(options.upstream_branch,
-                                            src.tgz,
-                                            src.additional_tarballs.items())
+        ret = 0
     except KeyboardInterrupt:
-        ret = 1
         gbp.log.err("Interrupted. Aborting.")
     except gbpc.CommandExecFailed:
-        ret = 1
+        pass  # command itself printed an error
     except GitRepositoryError as msg:
         gbp.log.err("Git command failed: %s" % msg)
-        ret = 1
     except GbpError as err:
         if str(err):
             gbp.log.err(err)
-        ret = 1
     except SkipImport:
         skipped = True
+        ret = 0
     finally:
         os.chdir(dirs['top'])
 
@@ -511,7 +548,7 @@ def main(argv):
             gbpc.RemoveTree(dirs[d])()
 
     if not ret and not skipped:
-        gbp.log.info("Version '%s' imported under '%s'" % (src.version, repo.path))
+        gbp.log.info("Version '%s' imported under '%s'" % (dsc.version, repo.path))
     return ret
 
 
