@@ -23,11 +23,21 @@ import unittest
 
 from gbp.command_wrappers import GitCommand
 from gbp.scripts.pq import (generate_patches, export_patches,
-                            import_quilt_patches, rebase_pq, SERIES_FILE)
+                            import_quilt_patches, rebase_pq,
+                            switch_pq,
+                            SERIES_FILE)
 import gbp.scripts.common.pq as pq
 import gbp.patch_series
 
 
+class TestPqOptions(object):
+    abbrev = 7
+    drop = False
+    patch_num_format = '%04d-'
+    patch_numbers = False
+    renumber = False
+
+
 class TestApplyAndCommit(testutils.DebianGitTestRepo):
     """Test L{gbp.pq}'s apply_and_commit"""
 
@@ -40,7 +50,16 @@ class TestApplyAndCommit(testutils.DebianGitTestRepo):
         patch = gbp.patch_series.Patch(_patch_path('foo.patch'))
 
         pq.apply_and_commit_patch(self.repo, patch, None)
-        self.assertIn('foo', self.repo.list_files())
+        self.assertIn(b'foo', self.repo.list_files())
+
+    def test_apply_and_commit_patch_preserve_subject(self):
+        """Test applying a patch preserves the subject"""
+        patch = gbp.patch_series.Patch(_patch_path('brackets-in-subject.patch'))
+
+        pq.apply_and_commit_patch(self.repo, patch, None)
+        self.assertIn(b'foo', self.repo.list_files())
+        info = self.repo.get_commit_info('HEAD')
+        self.assertEquals('[text] foobar', info['subject'])
 
     def test_topic(self):
         """Test if setting a topic works"""
@@ -61,8 +80,7 @@ class TestApplyAndCommit(testutils.DebianGitTestRepo):
     @unittest.skipIf(not os.path.exists('/usr/bin/dpkg'), 'Dpkg not found')
     def test_debian_missing_author(self):
         """
-        Check if we parse the author from debian control
-        if it's missing in the patch.
+        Check if we parse the author from debian control if it's missing in the patch.
         """
         def _check_log(msg):
             self.assertEqual(msg, "Patch 'foo.patch' has no authorship "
@@ -86,7 +104,7 @@ class TestApplyAndCommit(testutils.DebianGitTestRepo):
         gbp.log.warn = orig_warn
         info = self.repo.get_commit_info('HEAD')
         self.assertEqual(info['author'].email, 'gg@godiug.net')
-        self.assertIn('foo', self.repo.list_files())
+        self.assertIn(b'foo', self.repo.list_files())
 
 
 class TestApplySinglePatch(testutils.DebianGitTestRepo):
@@ -101,8 +119,9 @@ class TestApplySinglePatch(testutils.DebianGitTestRepo):
 
         patch = gbp.patch_series.Patch(_patch_path('foo.patch'))
 
+        self.repo.create_branch(pq.pq_branch_name('master'))
         pq.apply_single_patch(self.repo, 'master', patch, None)
-        self.assertIn('foo', self.repo.list_files())
+        self.assertIn(b'foo', self.repo.list_files())
 
 
 class TestWritePatch(testutils.DebianGitTestRepo):
@@ -140,16 +159,11 @@ class TestWritePatch(testutils.DebianGitTestRepo):
         self.repo.commit_all("foo")
         diff = self.repo.diff('master', 'testapply')
         # Branches must be identical afterwards
-        self.assertEqual('', diff)
+        self.assertEqual(b'', diff)
 
     def test_generate_patches(self):
         """Test generation of patches"""
 
-        class opts:
-            patch_numbers = False
-            renumber = False
-            patch_num_format = '%04d-'
-
         expected_patches = ['gbptest/added-foo.patch',
                             'gbptest/patchname.diff']
 
@@ -159,16 +173,13 @@ class TestWritePatch(testutils.DebianGitTestRepo):
                                    "Gbp-Pq: Topic gbptest\n"
                                    "Gbp-Pq: Name patchname.diff"))]
 
+        opts = TestPqOptions()
+        opts.patch_num_format = '%04d-'
         self._test_generate_patches(changes, expected_patches, opts)
 
     def test_generate_renumbered_patches(self):
         """Test generation of renumbered patches"""
 
-        class opts:
-            patch_numbers = True
-            renumber = True
-            patch_num_format = '%02d_'
-
         expected_patches = ['gbptest/01_added-foo.patch',
                             'gbptest/02_patchname.diff']
 
@@ -178,16 +189,15 @@ class TestWritePatch(testutils.DebianGitTestRepo):
                                    "Gbp-Pq: Topic gbptest\n"
                                    "Gbp-Pq: Name 099-patchname.diff"))]
 
+        opts = TestPqOptions()
+        opts.patch_num_format = '%02d_'
+        opts.renumber = True
+        opts.patch_numbers = True
         self._test_generate_patches(changes, expected_patches, opts)
 
     def test_generate_patches_with_name_clashes(self):
         """Test generation of patches which have name clashes"""
 
-        class opts:
-            patch_numbers = False
-            renumber = True
-            patch_num_format = '%02d_'
-
         expected_patches = ['gbptest/added-foo.patch',
                             'gbptest/patchname.diff',
                             'gbptest/patchname-1.diff',
@@ -205,19 +215,18 @@ class TestWritePatch(testutils.DebianGitTestRepo):
                                      "Gbp-Pq: Topic gbptest\n"
                                      "Gbp-Pq: Name 101-patchname.diff"))]
 
+        opts = TestPqOptions()
+        opts.renumber = True
+        opts.patch_num_format = '%02d_'
         self._test_generate_patches(changes, expected_patches, opts)
 
 
 class TestExport(testutils.DebianGitTestRepo):
-    class Options(object):
-        drop = False
-        patch_numbers = False
-        renumber = False
-        patch_num_format = ''
+    class Options(TestPqOptions):
+        commit = False
         meta_closes = False
         meta_closes_bugnum = ''
         pq_from = 'DEBIAN'
-        commit = False
 
     def setUp(self):
         testutils.DebianGitTestRepo.setUp(self)
@@ -231,7 +240,8 @@ class TestExport(testutils.DebianGitTestRepo):
         opts = TestExport.Options()
         opts.drop = True
 
-        pq.switch_pq(repo, start)
+        repo.create_branch(pq.pq_branch_name('master'))
+        switch_pq(repo, start, opts)
         self.assertEqual(repo.get_branch(), pq_branch)
         export_patches(repo, pq_branch, opts)
         self.assertEqual(repo.get_branch(), start)
@@ -244,7 +254,8 @@ class TestExport(testutils.DebianGitTestRepo):
         pq_branch = os.path.join('patch-queue', start)
         opts = TestExport.Options()
         opts.commit = True
-        pq.switch_pq(repo, start)
+        repo.create_branch(pq.pq_branch_name('master'))
+        switch_pq(repo, start, opts)
         self.assertEqual(len(repo.get_commits()), 1)
         self.assertEqual(repo.get_branch(), pq_branch)
         self.add_file('foo', 'foo')
@@ -272,7 +283,8 @@ class TestExport(testutils.DebianGitTestRepo):
             f.write("patch2.diff\n")
         repo.add_files('debian/patches')
         repo.commit_all('Add series file')
-        pq.switch_pq(repo, start)
+        repo.create_branch(pq.pq_branch_name('master'))
+        switch_pq(repo, start, opts)
         self.assertEqual(len(repo.get_commits()), 2)
         self.assertEqual(repo.get_branch(), pq_branch)
         export_patches(repo, pq_branch, opts)
@@ -280,8 +292,8 @@ class TestExport(testutils.DebianGitTestRepo):
         self.assertTrue(repo.has_branch(pq_branch))
         self.assertEqual(len(repo.get_commits()), 3,
                          "Export did not create commit")
-        self.assertIn("Dropped patch1.diff:", repo.show('HEAD'))
-        self.assertIn("Dropped patch2.diff:", repo.show('HEAD'))
+        self.assertIn(b"Drop patch1.diff:", repo.show('HEAD'))
+        self.assertIn(b"Drop patch2.diff:", repo.show('HEAD'))
 
 
 class TestParseGbpCommand(unittest.TestCase):
@@ -312,16 +324,12 @@ class TestParseGbpCommand(unittest.TestCase):
 class TestFromTAG(testutils.DebianGitTestRepo):
     """Test L{gbp.pq}'s pq-from=TAG"""
 
-    class Options(object):
+    class Options(TestPqOptions):
         commit = True
-        drop = False
         force = False
         meta_closes = 'Closes|LP'
         meta_closes_bugnum = r'(?:bug|issue)?\#?\s?\d+'
-        patch_num_format = '%04d-'
-        patch_numbers = False
         pq_from = 'TAG'
-        renumber = False
         upstream_tag = 'upstream/%(version)s'
 
     def git_create_empty_branch(self, branch):
@@ -355,24 +363,23 @@ class TestFromTAG(testutils.DebianGitTestRepo):
                              pq_from=TestFromTAG.Options.pq_from,
                              upstream_tag=TestFromTAG.Options.upstream_tag)
         diff = self.repo.diff(self.repo.get_branch(), 'upstream/0.0.1')
-        self.assertEqual('', diff)
+        self.assertEqual(b'', diff)
         diff = self.repo.diff(self.repo.get_branch(), 'master')
-        self.assertNotEqual('', diff)
+        self.assertNotEqual(b'', diff)
 
         rebase_pq(self.repo,
                   branch=self.repo.get_branch(),
-                  pq_from=TestFromTAG.Options.pq_from,
-                  upstream_tag=TestFromTAG.Options.upstream_tag)
+                  options=TestFromTAG.Options)
         diff = self.repo.diff(self.repo.get_branch(), 'upstream/0.0.1')
-        self.assertEqual('', diff)
+        self.assertEqual(b'', diff)
         diff = self.repo.diff(self.repo.get_branch(), 'master')
-        self.assertNotEqual('', diff)
+        self.assertNotEqual(b'', diff)
 
         export_patches(self.repo,
                        branch=self.repo.get_branch(),
                        options=TestFromTAG.Options())
         diff = self.repo.diff(self.repo.get_branch(), self.repo.head)
-        self.assertEqual('', diff)
+        self.assertEqual(b'', diff)
 
     def test_adding_patch(self):
         import_quilt_patches(self.repo,
@@ -389,10 +396,10 @@ class TestFromTAG(testutils.DebianGitTestRepo):
         self.assertTrue(os.path.exists(os.path.join(self.repo.path,
                                                     os.path.dirname(SERIES_FILE),
                                                     'added-bar.patch')))
+        switch_pq(self.repo, 'master', TestFromTAG.Options)
         rebase_pq(self.repo,
                   branch=self.repo.get_branch(),
-                  pq_from=TestFromTAG.Options.pq_from,
-                  upstream_tag=TestFromTAG.Options.upstream_tag)
+                  options=TestFromTAG.Options())
         export_patches(self.repo,
                        branch=self.repo.get_branch(),
                        options=TestFromTAG.Options())
@@ -413,10 +420,10 @@ class TestFromTAG(testutils.DebianGitTestRepo):
             ' -- Mr. T. S. <t@example.com>  '
             'Thu, 01 Jan 1970 00:00:00 +0000\n'
         )
+        switch_pq(self.repo, 'master', TestFromTAG.Options())
         rebase_pq(self.repo,
                   branch=self.repo.get_branch(),
-                  pq_from=TestFromTAG.Options.pq_from,
-                  upstream_tag=TestFromTAG.Options.upstream_tag)
+                  options=TestFromTAG.Options())
         export_patches(self.repo,
                        branch=self.repo.get_branch(),
                        options=TestFromTAG.Options())
