@@ -30,6 +30,8 @@ from gbp.scripts.import_orig import main as import_orig
 from gbp.deb.pristinetar import DebianPristineTar
 from gbp.deb.dscfile import DscFile
 from gbp.git.repository import GitRepository, GitRepositoryError
+from gbp.paths import to_bin
+from gbp.command_wrappers import UnpackTarArchive
 
 from nose.tools import ok_, eq_, assert_raises
 
@@ -85,9 +87,9 @@ class TestImportOrig(ComponentTestBase):
 
     def _check_component_tarballs(self, repo, files):
         for file in files:
-            ok_(file in repo.ls_tree('HEAD'),
+            ok_(to_bin(file) in repo.ls_tree('HEAD'),
                 "Could not find component tarball file %s in %s" % (file, repo.ls_tree('HEAD')))
-            ok_(file in repo.ls_tree('upstream'),
+            ok_(to_bin(file) in repo.ls_tree('upstream'),
                 "Could not find component tarball file %s in %s" % (file, repo.ls_tree('upstream')))
 
     @RepoFixtures.quilt30(DEFAULT_DSC, opts=['--pristine-tar'])
@@ -101,7 +103,8 @@ class TestImportOrig(ComponentTestBase):
                          '--no-interactive', '--pristine-tar', orig]) == 0)
         self._check_repo_state(repo, 'master', ['master', 'upstream', 'pristine-tar'],
                                tags=['debian/2.6-2', 'upstream/2.6', 'upstream/2.8'])
-        eq_(os.path.exists('postimport.out'), True)
+        ok_(os.path.exists('debian/changelog'))
+        ok_(os.path.exists('postimport.out'))
         self.check_hook_vars('postimport', [("GBP_BRANCH", "master"),
                                             ("GBP_TAG", "upstream/2.8"),
                                             ("GBP_UPSTREAM_VERSION", "2.8"),
@@ -117,7 +120,8 @@ class TestImportOrig(ComponentTestBase):
         ok_(import_orig(['arg0', '--component=foo', '--no-interactive', '--pristine-tar', orig]) == 0)
         self._check_repo_state(repo, 'master', ['master', 'upstream', 'pristine-tar'],
                                tags=['debian/2.6-2', 'upstream/2.6', 'upstream/2.8'])
-        self._check_component_tarballs(repo, ['foo/test1', 'foo/test2'])
+        self._check_component_tarballs(repo, [b'foo/test1', b'foo/test2'])
+        ok_(os.path.exists('debian/changelog'))
 
         dsc = DscFile.parse(_dsc_file(self.pkg, '2.8-1', dir='dsc-3.0-additional-tarballs'))
         # Check if we can rebuild the upstream tarball and additional tarball
@@ -142,6 +146,7 @@ class TestImportOrig(ComponentTestBase):
         self._check_repo_state(repo, 'master', ['master', 'upstream', 'pristine-tar'],
                                tags=['debian/2.6-2', 'upstream/2.6', 'upstream/2.8', 'upstream/2.9'])
         self._check_component_tarballs(repo, ['foo/test1', 'foo/test2', 'foo/test3'])
+        ok_(os.path.exists('debian/changelog'))
 
         dsc = DscFile.parse(_dsc_file(self.pkg, '2.9-1', dir='dsc-3.0-additional-tarballs'))
         # Check if we can rebuild the upstream tarball and additional tarball
@@ -233,7 +238,7 @@ class TestImportOrig(ComponentTestBase):
     @RepoFixtures.quilt30(DEFAULT_DSC, opts=['--pristine-tar'])
     def test_filter_with_component_tarballs(self, repo):
         """
-        Test that using a filter works component tarballs (#840602)
+        Test that using a filter works with component tarballs (#840602)
         """
         # copy data since we don't want the repacked tarball to end up in DEB_TEST_DATA_DIR
         os.mkdir('../tarballs')
@@ -300,6 +305,39 @@ class TestImportOrig(ComponentTestBase):
                 t.getmember(f)
         t.close()
 
+    @RepoFixtures.quilt30(DEFAULT_DSC, opts=['--pristine-tar'])
+    def test_filter_unpacked_dir(self, repo):
+        """
+        Test that importing and filtering unpacked upstream source works.
+        """
+        f = 'hello-debhelper_2.8.orig.tar.gz'
+        src = os.path.join(DEB_TEST_DATA_DIR, 'dsc-3.0', f)
+
+        # Create an unpacked tarball we can import
+        UnpackTarArchive(src, '..')()
+        ok_(os.path.exists('../hello-2.8'))
+
+        ok_(import_orig(['arg0',
+                         '--no-interactive',
+                         '--pristine-tar',
+                         '--filter-pristine-tar',
+                         '--filter=README*',
+                         '../hello-2.8']) == 0)
+        self._check_repo_state(repo, 'master', ['master', 'upstream', 'pristine-tar'],
+                               tags=['debian/2.6-2', 'upstream/2.6', 'upstream/2.8'])
+
+        filtered = os.path.join('..', f)
+        ok_(os.path.exists(filtered))
+        # Check if tar got filtered properly
+        t = tarfile.open(name=filtered, mode="r:gz")
+        for f in ['hello-2.8/configure']:
+            i = t.getmember(f)
+            eq_(type(i), tarfile.TarInfo)
+        for f in ['hello-2.8/README']:
+            with assert_raises(KeyError):
+                t.getmember(f)
+        t.close()
+
     @RepoFixtures.quilt30(DEFAULT_DSC, opts=['--pristine-tar'])
     def test_import_in_submodule(self, repo):
         """
